
import subprocess
import os

sVersion = '3.8.0'

try:
    from vsg import version_info
    try:
        sVersion = version_info.sVersion
    except AttributeError:
        pass
    sShaNum = version_info.sShaNum
except ImportError:
    sShaNum = None


def print_version(oCommandLineArguments, sVersion=sVersion, sShaNum=sShaNum):

    if (oCommandLineArguments.version):

        sVersion, sShaNum = get_version_info(sVersion, sShaNum)

        print('VHDL Style Guide (VSG) version: ' + str(sVersion))

        print('Git commit SHA: ' + str(sShaNum))

        exit(0)

def get_version_info(sVersion, sShaNum):

    if sVersion is not None and sShaNum is not None:
        return sVersion, sShaNum

    sReturnPath = os.getcwd()
    sPath = os.path.dirname(__file__)
    if not os.path.isdir(os.path.join(sPath, '..', '.git')):
        return sVersion + '+zip.file', 'Unknown.  Installed via zip file.'

    try:
        os.chdir(sPath)
        lActual = subprocess.check_output(['git', 'describe', '--tags'])
        lActual = str(lActual.decode('utf-8')).split('\n')
        lVersion = lActual[0].split('-')
        try:
            sVersion = str(lVersion[0]) + '.dev' + str(lVersion[1])
            sShaNum = str(lVersion[-1])
        except IndexError:
            sVersion = str(lVersion[0])
        os.chdir(sReturnPath)
        return sVersion, sShaNum
    except subprocess.CalledProcessError:
        os.chdir(sReturnPath)
        return sVersion + '+zip.file', 'Unknown.  Installed via zip file.'




from vsg import token

from vsg.rules import single_space_after_token

lTokens = []
lTokens.append(token.variable_declaration.colon)


class rule_005(single_space_after_token):
    '''
    This rule checks there is a single space after the colon.

    **Violation**

    .. code-block:: vhdl

       variable count   :integer;
       variable counter :     integer;

    **Fix**

    .. code-block:: vhdl

       variable count   : integer;
       variable counter : integer;
    '''
    def __init__(self):
        single_space_after_token.__init__(self, 'variable', '005', lTokens)
        self.solution = 'Ensure only a single space after the colon.'




from vsg.rules import single_space_between_tokens

from vsg.token import loop_statement as token


class rule_101(single_space_between_tokens):
    '''
    This rule checks for a single space before the ending loop label if it exists.

    **Violation**

    .. code-block:: vhdl

       end loop           END_LOOP_LABEL;

    **Fix**

    .. code-block:: vhdl

       end loop END_LOOP_LABEL;
    '''
    def __init__(self):
        single_space_between_tokens.__init__(self, 'loop_statement', '101', token.end_loop_keyword, token.end_loop_label)
        self.solution = 'Reduce spaces between **loop** keyword and label to single space.'




import string

from vsg import block_rule
from vsg import parser
from vsg import violation

from vsg.vhdlFile import utils


class rule_002(block_rule.Rule):
    '''
    This rule checks the **comment_left** attribute exists for all comments.

    |configuring_block_comments_link|

    **Violation**

    .. code-block:: vhdl

       --+-------------[ Header ]==============
       --   Comment
       --   Comment
       ----------------------------------------

    **Fix**

    .. code-block:: vhdl

       --+-------------[ Header ]==============
       --|  Comment
       --|  Comment
       ----------------------------------------
    '''

    def __init__(self):
        block_rule.Rule.__init__(self, 'block_comment', '002')
        self.comment_left = None
        self.configuration.append('comment_left')

    def analyze(self, oFile):

        self._print_debug_message('Analyzing rule: ' + self.unique_id)
        lToi = self._get_tokens_of_interest(oFile)

        lUpdate = []

        for oToi in lToi:
            iLine, lTokens = utils.get_toi_parameters(oToi)

            iComments = utils.count_token_types_in_list_of_tokens(parser.comment, lTokens)

            iComment = 0
            for iToken, oToken in enumerate(lTokens):
                iLine = utils.increment_line_number(iLine, oToken)

                if isinstance(oToken, parser.comment):
                    iComment += 1
                    if iComment == 1:
                        if not is_header(oToken.get_value()):
                            break
                    elif iComment > 1 and iComment < iComments:
                        if not self.allow_indenting:
                            oToken.set_indent(0)

                        if self.comment_left is None:
                            continue

                        if isinstance(lTokens[iToken - 1], parser.whitespace):
                            if not self.allow_indenting:
                                break

                        sHeader = '--'
                        sHeader += self.comment_left
                        sComment = oToken.get_value()
                        if not sComment.startswith(sHeader):
                            sSolution = 'Comment must start with ' + sHeader
                            oViolation = violation.New(iLine, oToi, sSolution)
                            self.add_violation(oViolation)

            if not self.allow_indenting:
                lUpdate.append(violation.New(0, oToi, ''))

        if not self.allow_indenting:
            oFile.update(lUpdate)


def is_header(sComment):
    try:
        if sComment[2] not in string.punctuation:
            return False
        if sComment[2] == '!':
            return False
        if sComment[3] not in string.punctuation:
            return False
    except IndexError:
        return True
    return True




from vsg.rules import insert_carriage_return_after_token_if_it_is_not_followed_by_a_comment_when_between_tokens

from vsg import token


class rule_013(insert_carriage_return_after_token_if_it_is_not_followed_by_a_comment_when_between_tokens):
    '''
    This rule checks for multiple ports declared on a single line.

    **Violation**

    .. code-block:: vhdl

       port (
         WR_EN    : in    std_logic;RD_EN    : in    std_logic;
         OVERFLOW : out   std_logic;DATA     : inout std_logic
       );

    **Fix**

    .. code-block:: vhdl

       port (
         WR_EN    : in    std_logic;
         RD_EN    : in    std_logic;
         OVERFLOW : out   std_logic;
         DATA     : inout std_logic
       );
    '''

    def __init__(self):
        insert_carriage_return_after_token_if_it_is_not_followed_by_a_comment_when_between_tokens.__init__(self, 'port', '013', [token.interface_list.semicolon], token.port_clause.open_parenthesis, token.port_clause.close_parenthesis)
        self.solution = 'Move multiple ports to their own lines.'




import string

from vsg import block_rule
from vsg import parser
from vsg import violation

from vsg.vhdlFile import utils


class rule_003(block_rule.Rule):
    '''
    This rule checks the block comment footer is correct.

    |configuring_block_comments_link|

    **Violation**

    .. code-block:: vhdl

       --+-------------[ Header ]==============
       --|  Comment
       --|  Comment
       ----------------------------------------

    **Fix**

    .. code-block:: vhdl

       --+-------------[ Header ]==============
       --|  Comment
       --|  Comment
       --+--------------------------[ Footer ]=
    '''

    def __init__(self):
        block_rule.Rule.__init__(self, 'block_comment', '003')
        self.footer_left = None
        self.footer_left_repeat = '-'
        self.footer_string = None
        self.footer_right_repeat = None
        self.footer_alignment = 'center'
        self.max_footer_column = 120
        self.configuration.extend(['footer_left', 'footer_left_repeat', 'footer_string', 'footer_right_repeat', 'footer_alignment', 'max_footer_column'])

    def analyze(self, oFile):

        self._print_debug_message('Analyzing rule: ' + self.unique_id)
        lToi = self._get_tokens_of_interest(oFile)

        lUpdate = []

        for oToi in lToi:
            iLine, lTokens = utils.get_toi_parameters(oToi)

            iComments = utils.count_token_types_in_list_of_tokens(parser.comment, lTokens)

            iComment = 0
            for iToken, oToken in enumerate(lTokens):
                iLine = utils.increment_line_number(iLine, oToken)

                if isinstance(oToken, parser.comment):
                    iComment += 1
                    if iComment == iComments:

                        if not self.allow_indenting:
                            if isinstance(lTokens[iToken - 1], parser.whitespace):
                                break
                            else:
                                oToken.set_indent(0)

                        iWhitespace = self.indentSize * oToken.get_indent()

                        sFooter = '--'
                        if self.footer_left is not None:
                            sFooter += self.footer_left
                            iFooter_left = len(self.footer_left)
                        else:
                            iFooter_left = 0

                        if self.footer_string is None:
                            sFooter += self.footer_left_repeat * (self.max_footer_column - iWhitespace - len(sFooter))
                        elif self.footer_alignment == 'center':
                            iLength = int((self.max_footer_column - iWhitespace - len(self.footer_string)) / 2) - iFooter_left - 2
                            sFooter += self.footer_left_repeat * (iLength)
                            sFooter += self.footer_string
                            sFooter += self.footer_right_repeat * (self.max_footer_column - len(sFooter))
                        elif self.footer_alignment == 'left':
                            sFooter += self.footer_left_repeat
                            sFooter += self.footer_string
                            iLength = self.max_footer_column - iWhitespace - len(sFooter)
                            sFooter += self.footer_right_repeat * (self.max_footer_column - len(sFooter))
                        elif self.footer_alignment == 'right':
                            iLength = self.max_footer_column - iWhitespace - len(sFooter) - len(self.footer_string) - 1
                            sFooter += self.footer_left_repeat * (iLength)
                            sFooter += self.footer_string
                            sFooter += self.footer_right_repeat

                        sComment = oToken.get_value()
                        try:
                            if is_footer(sComment):
                                if not self.allow_indenting:
                                    oToken.set_indent(0)
                                if sComment != sFooter:
                                    sSolution = 'Change block comment footer to : ' + sFooter
                                    oViolation = violation.New(iLine, oToi, sSolution)
                                    self.add_violation(oViolation)
                                    break
                        except IndexError:
                            break

            if not self.allow_indenting:
                lUpdate.append(violation.New(0, oToi, ''))

        if not self.allow_indenting:
            oFile.update(lUpdate)


def is_footer(sComment):
    try:
        if sComment[2] not in string.punctuation:
            return False
        if sComment[2] == '!':
            return False
        if sComment[3] not in string.punctuation:
            return False
        return True
    except IndexError:
        return True
    return True




from vsg import parser
from vsg import violation

from vsg.rule_group import indent
from vsg.rules import utils as rules_utils


class token_indent_unless_between_tokens(indent.Rule):
    '''
    Checks the case for words.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.
    '''

    def __init__(self, name, identifier, lTokens, lUnless):
        indent.Rule.__init__(self, name=name, identifier=identifier)
        self.lTokens = lTokens
        self.lUnless = lUnless

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_tokens_at_beginning_of_line_matching_unless_between_tokens(self.lTokens, self.lUnless)

    def _analyze(self, lToi):
        for oToi in lToi:
            lTokens = oToi.get_tokens()

            if len(lTokens) == 2 and lTokens[1].get_indent() == 0:
                sSolution = "Indent level 0"
                oViolation = violation.New(oToi.get_line_number(), oToi, sSolution)
                oViolation.set_action('remove_whitespace')
                self.add_violation(oViolation)
            elif len(lTokens) == 2:
                iWhitespace = len(lTokens[0].get_value())
                iIndent = self.indentSize * lTokens[1].get_indent()
                if iWhitespace != iIndent:
                    sSolution = 'Indent level ' + str(lTokens[1].get_indent())
                    oViolation = violation.New(oToi.get_line_number(), oToi, sSolution)
                    oViolation.set_action('adjust_whitespace')
                    self.add_violation(oViolation)
            elif len(lTokens) == 1:
                if lTokens[0].get_indent() != 0:
                    sSolution = 'Indent level ' + str(lTokens[0].get_indent())
                    oViolation = violation.New(oToi.get_line_number(), oToi, sSolution)
                    oViolation.set_action('add_whitespace')
                    self.add_violation(oViolation)

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()
        if oViolation.get_action() == 'remove_whitespace':
            oViolation.set_tokens([lTokens[1]])
        elif oViolation.get_action() == 'adjust_whitespace':
            lTokens[0].set_value(lTokens[1].get_indent() * self.indentSize * ' ')
            oViolation.set_tokens(lTokens)
        elif oViolation.get_action() == 'add_whitespace':
            rules_utils.insert_whitespace(lTokens, 0, lTokens[0].get_indent() * self.indentSize)
            oViolation.set_tokens(lTokens)




from vsg import token

from vsg.rules import single_space_between_token_pairs

lTokens = []
lTokens.append([token.process_statement.process_keyword, token.process_statement.open_parenthesis])
lTokens.append([token.process_statement.process_keyword, token.process_statement.is_keyword])
lTokens.append([token.process_statement.process_keyword, token.process_statement.begin_keyword])


class rule_002(single_space_between_token_pairs):
    '''
    This rule checks for a single space after the **process** keyword.

    **Violation**

    .. code-block:: vhdl

       proc_a : process(rd_en, wr_en, data_in, data_out,

       proc_a : process    (rd_en, wr_en, data_in, data_out,

    **Fix**

    .. code-block:: vhdl

       proc_a : process (rd_en, wr_en, data_in, data_out,

       proc_a : process (rd_en, wr_en, data_in, data_out,
    '''
    def __init__(self):
        single_space_between_token_pairs.__init__(self, 'process', '002', lTokens)
        self.solution = 'Ensure a single space after the *process* keyword.'




from vsg.rules import single_space_between_tokens

from vsg.token import context_reference as token


class rule_002(single_space_between_tokens):
    '''
    This rule checks for a single space between the **context** keyword and the context selected name.

    **Violation**

    .. code-block:: vhdl

       context   c1;

    **Fix**

    .. code-block:: vhdl

       context c1;
    '''
    def __init__(self):
        single_space_between_tokens.__init__(self, 'context_ref', '002', token.keyword, token.selected_name)
        self.solution = 'Reduce spaces between *context* keyword and selected_name.'




from vsg import parser

from vsg.rules import whitespace_before_token

lTokens = []
lTokens.append(parser.comment)


class rule_004(whitespace_before_token):
    '''
    This rule checks for at least a single space before inline comments.

    **Violation**

    .. code-block:: vhdl

       wr_en <= '1';--Write data
       rd_en <= '1';   -- Read data

    **Fix**

    .. code-block:: vhdl

       wr_en <= '1'; --Write data
       rd_en <= '1';   -- Read data
    '''

    def __init__(self):
        whitespace_before_token.__init__(self, 'comment', '004', lTokens)
        self.solution = 'Add a single space before comment'




from vsg import token

from vsg.rules import single_space_between_token_pairs

lTokens = []
lTokens.append([token.package_declaration.end_keyword, token.package_declaration.end_package_keyword])
lTokens.append([token.package_declaration.end_package_keyword, token.package_declaration.end_package_simple_name])


class rule_009(single_space_between_token_pairs):
    '''
    This rule checks for a single space between the **end** and **package** keywords and package name.

    **Violation**

    .. code-block:: vhdl

       end   package   FIFO_PKG;

    **Fix**

    .. code-block:: vhdl

       end package FIFO_PKG;
    '''
    def __init__(self):
        single_space_between_token_pairs.__init__(self, 'package', '009', lTokens)
        self.solution = 'Single space between *end* and *package* keywords and package_simple_name.'




import bisect
import pprint

from vsg import parser


class New():

    def __init__(self, dMap):
        self.dMap = dMap

    def get_token_indexes(self, oToken, bCopy=False):
        sBase, sSub = extract_unique_id(oToken)
        try:
            if bCopy:
                return self.dMap[sBase][sSub].copy()
            else:
                return self.dMap[sBase][sSub]
        except KeyError:
            return []

    def get_token_indexes_between_indexes(self, oToken, iStart, iEnd):
        lReturn = []
        lIndexes = self.get_token_indexes(oToken)
        for iIndex in lIndexes:
            if iIndex > iStart and iIndex < iEnd:
                lReturn.append(iIndex)
        return lReturn

    def get_line_number_of_index(self, iIndex):
        iLine = bisect.bisect_left(self.dMap['parser']['carriage_return'], iIndex) + 1
        return iLine

    def get_index_of_carriage_return_after_index(self, iIndex):
        iTemp = bisect.bisect_right(self.dMap['parser']['carriage_return'], iIndex)
        return self.dMap['parser']['carriage_return'][iTemp]

    def get_index_of_carriage_return_before_index(self, iIndex):
        if iIndex == 0:
            return None
        iTemp = bisect.bisect_left(self.dMap['parser']['carriage_return'], iIndex) - 1
        return self.dMap['parser']['carriage_return'][iTemp]

    def get_index_of_token_after_index(self, oToken, iIndex):
        sBase, sSub = extract_unique_id(oToken)
        try:
            iTemp = bisect.bisect_right(self.dMap[sBase][sSub], iIndex)
            return self.dMap[sBase][sSub][iTemp]
        except IndexError:
            return None
        except KeyError:
            return None

    def get_token_pair_indexes(self, oStart, oEnd):
        lStartIndexes = self.get_token_indexes(oStart)
        lReturnEndIndexes = []
        lReturnStartIndexes = []
        for iIndex in lStartIndexes:
            iEndIndex = self.get_index_of_token_after_index(oEnd, iIndex)
            if iEndIndex is not None:
                lReturnStartIndexes.append(iIndex)
                lReturnEndIndexes.append(iEndIndex)
        return lReturnStartIndexes, lReturnEndIndexes

    def get_index_of_next_non_whitespace_token(self, iIndex, bExcludeComments=False):
        iStartIndex = iIndex + 1
        lTokens = [None, None, None, None]
        lBaseKeys = list(self.dMap.keys())
        for sBaseKey in lBaseKeys:
            lSubKeys = list(self.dMap[sBaseKey].keys())
            if sBaseKey == 'parser':
                lSubKeys.remove('whitespace')
                lSubKeys.remove('carriage_return')
                lSubKeys.remove('blank_line')
            try:
                if bExcludeComments:
                    lSubKeys.remove('comment')
            except ValueError:
                pass
            for sSubKey in lSubKeys:
                for iIdx in range(0, 4):
                    iSearchIdx = iStartIndex + iIdx
                    if iSearchIdx in self.dMap[sBaseKey][sSubKey]:
                        lTokens[iIdx] = iSearchIdx
                        continue

        for iToken in lTokens:
            if iToken is not None:
                return iToken

    def get_index_of_previous_non_whitespace_token_before_index(self, iIndex):
        iStartIndex = iIndex - 1
        for i in range(iStartIndex, 0, -1):
            if not self.is_token_at_index_whitespace(i):
                return i

    def get_index_of_previous_non_whitespace_token(self, iIndex):
        iStartIndex = iIndex - 1
        for i in range(iStartIndex, 0, -1):
            if not self.is_token_at_index_whitespace_or_comment(i):
                return i

    def is_token_at_index(self, oToken, iIndex):
        try:
            sBase, sSub = extract_unique_id(oToken)
            if iIndex in self.dMap[sBase][sSub]:
                return True
            return False
        except KeyError:
            return False

    def is_token_at_index_whitespace(self, iIndex):
        if self.is_token_at_index(parser.whitespace, iIndex):
            return True
        if self.is_token_at_index(parser.carriage_return, iIndex):
            return True
        if self.is_token_at_index(parser.blank_line, iIndex):
            return True
        return False

    def is_token_at_index_whitespace_or_comment(self, iIndex):
        if self.is_token_at_index(parser.whitespace, iIndex):
            return True
        if self.is_token_at_index(parser.comment, iIndex):
            return True
        if self.is_token_at_index(parser.carriage_return, iIndex):
            return True
        if self.is_token_at_index(parser.blank_line, iIndex):
            return True
        return False

    def pretty_print(self):
        pp=pprint.PrettyPrinter(indent=4)
        pp.pprint(self.dMap)


def extract_unique_id(oToken):
    lDoc = oToken.__doc__.split()
    for iDoc, sDoc in enumerate(lDoc):
        if sDoc == 'unique_id':
            return lDoc[iDoc + 2], lDoc[iDoc + 4]
    return None, None


def process_tokens(lTokens):
    dMap = build_default_map()
    for iToken, oToken in enumerate(lTokens):
        sBase, sSub = oToken.get_unique_id()
        if sBase is not None:
           try:
               dMap[sBase][sSub].append(iToken)
           except KeyError:
               try:
                   dMap[sBase][sSub] = []
                   dMap[sBase][sSub].append(iToken)
               except KeyError:
                   dMap[sBase] = {}
                   dMap[sBase][sSub] = []
                   dMap[sBase][sSub].append(iToken)
        if sBase == 'logical_operator':
            try:
                dMap[sBase][sBase].append(iToken)
            except KeyError:
                try:
                    dMap[sBase][sBase] = []
                    dMap[sBase][sBase].append(iToken)
                except KeyError:
                    dMap[sBase] = {}
                    dMap[sBase][sBase] = []
                    dMap[sBase][sBase].append(iToken)
            continue
        if sSub == 'comma':
            try:
                if iToken not in dMap['parser']['comma']:
                    dMap['parser']['comma'].append(iToken)
            except KeyError:
                try:
                    dMap['parser']['comma'] = []
                    dMap['parser']['comma'].append(iToken)
                except KeyError:
                    dMap['parser'] = {}
                    dMap['parser']['comma'] = []
                    dMap['parser']['comma'].append(iToken)
            continue
        if sSub == 'open_parenthesis':
            try:
                if iToken not in dMap['parser']['open_parenthesis']:
                    dMap['parser']['open_parenthesis'].append(iToken)
            except KeyError:
                try:
                    dMap['parser']['open_parenthesis'] = []
                    dMap['parser']['open_parenthesis'].append(iToken)
                except KeyError:
                    dMap['parser'] = {}
                    dMap['parser']['open_parenthesis'] = []
                    dMap['parser']['open_parenthesis'].append(iToken)
            continue

    return New(dMap)


def build_default_map():
    dMap = {}
    return dMap




from vsg.vhdlFile.extract import tokens
from vsg.vhdlFile.extract import utils


def get_consecutive_lines_starting_with_token(search_token, min_num_lines, lAllTokens, oTokenMap):

    lReturn = []
    lSearchIndexes = utils.filter_tokens_which_start_a_line(search_token, oTokenMap)

    lSearchLines = []
    for iSearchIndex in lSearchIndexes:
        lSearchLines.append(oTokenMap.get_line_number_of_index(iSearchIndex))

    iStart = None
    for iIndex, iLine in enumerate(lSearchLines):

        if iStart is None:
            iStart = iLine
            iStartLine = iLine
            iStartIndex = iIndex
            iCurrent = iLine
        else:
            if iLine == iCurrent + 1:
                iCurrent = iLine
                iEndIndex = iIndex
            else:
                if lSearchLines[iIndex - 1] - iStartLine >= min_num_lines - 1:
                    iStartToken = oTokenMap.get_index_of_carriage_return_before_index(lSearchIndexes[iStartIndex]) + 1
                    iEndToken = oTokenMap.get_index_of_carriage_return_after_index(lSearchIndexes[iEndIndex])
                    lTemp = lAllTokens[iStartToken:iEndToken]
                    lReturn.append(tokens.New(iStartToken, iStartLine, lTemp))
                iStart = iLine
                iStartLine = iLine
                iStartIndex = iIndex
                iCurrent = iLine

    if lSearchLines[iIndex - 1] - iStartLine >= min_num_lines - 1:
        iStartToken = oTokenMap.get_index_of_carriage_return_before_index(lSearchIndexes[iStartIndex]) + 1
        iEndToken = oTokenMap.get_index_of_carriage_return_after_index(lSearchIndexes[iEndIndex])
        lTemp = lAllTokens[iStartToken:iEndToken]
        lReturn.append(tokens.New(iStartToken, iStartLine, lTemp))

    return lReturn




from vsg.rules import token_case

from vsg.token import architecture_body as token


class rule_020(token_case):
    '''
    This rule checks the proper case of the **is** keyword in the architecture declaration.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       architecture rtl of fifo IS

    **Fix**

    .. code-block:: vhdl

       architecture rtl of fifo is
    '''

    def __init__(self):
        token_case.__init__(self, 'architecture', '020', [token.is_keyword])
        self.groups.append('case::keyword')




from vsg.rules import single_space_between_tokens

from vsg.token import context_declaration as token


class rule_019(single_space_between_tokens):
    '''
    This rule checks for a single space between the **context** keyword and the context identifier.

    **Violation**

    .. code-block:: vhdl

       end context;

       end context    c1;

    **Fix**

    .. code-block:: vhdl

       end context;

       end context c1;
    '''
    def __init__(self):
        single_space_between_tokens.__init__(self, 'context', '019', token.end_context_keyword, token.context_simple_name)
        self.solution = 'Reduce spaces between *context* keyword and context_simple_name to a single space.'




from vsg import token

from vsg.rules import whitespace_before_token

lTokens = []
lTokens.append(token.variable_declaration.colon)


class rule_006(whitespace_before_token):
    '''
    This rule checks for at least a single space before the colon.

    **Violation**

    .. code-block:: vhdl

       variable count: integer;
       variable counter : integer;

    **Fix**

    .. code-block:: vhdl

       variable count : integer;
       variable counter : integer;
    '''
    def __init__(self):
        whitespace_before_token.__init__(self, 'variable', '006', lTokens)
        self.solution = 'Ensure at least a single space exists before the :'




from vsg import parser

from vsg.rule_group import structure


class Rule(structure.Rule):

    def __init__(self, name, identifier):
        structure.Rule.__init__(self, name=name, identifier=identifier)
        self.fixable = False
        self.disable = True

        self.min_height = 3
        self.configuration.append('min_height')
        self.allow_indenting = True
        self.configuration.append('allow_indenting')

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_consecutive_lines_starting_with_token(parser.comment, self.min_height)

    def fix(self, oFile, dFixOnly=None):
        '''
        Applies fixes for any rule violations.
        '''
        self.analyze(oFile)





from vsg.token import signal_declaration as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import expression
from vsg.vhdlFile.classify import identifier_list
from vsg.vhdlFile.classify import signal_kind
from vsg.vhdlFile.classify import subtype_indication


def detect(iToken, lObjects):
    '''
    signal_declaration ::=
        signal identifier_list : subtype_indication [ signal_kind ] [ := expression ] ;
    '''

    if utils.is_next_token('signal', iToken, lObjects):
        return classify(iToken, lObjects)

    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.assign_next_token_required('signal', token.signal_keyword, iToken, lObjects)
    iCurrent = identifier_list.classify_until([':'], iCurrent, lObjects, token.identifier)
    iCurrent = utils.assign_next_token_required(':', token.colon, iCurrent, lObjects)

    iCurrent = subtype_indication.classify_until([';', ':=', 'bus', 'register'], iCurrent, lObjects)

    iCurrent = signal_kind.detect(iToken, lObjects)

    if utils.is_next_token(':=', iCurrent, lObjects):
        iCurrent = utils.assign_next_token_required(':=', token.assignment_operator, iCurrent, lObjects)
        iCurrent = expression.classify_until([';'], iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    return iCurrent




from vsg import parser

from vsg.vhdlFile.extract import tokens


def get_tokens_bounded_by(oStart, oEnd, lAllObjects, oTokenMap, include_trailing_whitespace=False, bExcludeLastToken=False, bIncludeTillEndOfLine=False, bIncludeTillBeginningOfLine=False):

#    oTokenMap.pretty_print()
    lReturn = []

    lStart, lEnd = oTokenMap.get_token_pair_indexes(oStart, oEnd)

    lCarriageReturns = oTokenMap.get_token_indexes(parser.carriage_return)

    lNewStart = []
    if bIncludeTillBeginningOfLine:
        for iStart in lStart:
            lNewStart.append(oTokenMap.get_index_of_carriage_return_before_index(iStart) + 1)
    else:
        lNewStart = lStart

    lNewEnd = []
    if bIncludeTillEndOfLine:
        for iEnd in lEnd:
            lNewEnd.append(oTokenMap.get_index_of_carriage_return_after_index(iEnd))
    else:
        lNewEnd = lEnd

    if bExcludeLastToken:
        lNewEnd = [x - 1 for x in lNewEnd]

#    print(lStart)
#    print(lEnd)
#    print(lNewEnd)

    lWhiteSpace = oTokenMap.get_token_indexes(parser.whitespace)
    if include_trailing_whitespace:
        for iNewEnd, iIndex in enumerate(lNewEnd):
            if iIndex + 1 in lWhiteSpace:
                lNewEnd[iNewEnd] +=1
    elif not bIncludeTillEndOfLine:
        for iNewEnd, iIndex in enumerate(lNewEnd):
            if iIndex in lWhiteSpace:
                lNewEnd[iNewEnd] -= 1
            if lNewEnd[iNewEnd] in lCarriageReturns:
                lNewEnd[iNewEnd] -= 1

#    print(lStart)
#    print(lEnd)
#    print(lNewEnd)

#    for i in range(0, len(lStart)):
#        iStart = lStart[i]
#        iEnd = lEnd[i]
#        iNewEnd = lNewEnd[i]
#        print(f'{lAllObjects[iStart]} | {lAllObjects[iEnd]} | {lAllObjects[iNewEnd]}')

    for iStart, iEnd, iIndex in zip(lNewStart, lNewEnd, lEnd):
        lTemp = lAllObjects[iStart: iEnd + 1]
        iStartLine = oTokenMap.get_line_number_of_index(iStart)
        oToi = tokens.New(iStart, iStartLine, lTemp)
        oToi.set_token_value(iIndex - iStart)
        lReturn.append(oToi)

    return lReturn

#    iLine = 1
#    lTemp = []
#    lReturn = []
#    bStore = False
#    bRightFound = False
#    for iIndex in range(0, len(lAllObjects)):
#        if isinstance(lAllObjects[iIndex], oLeft):
#            bStore = True
#            iStart = iIndex
#            iStartLine = iLine
#        if bStore:
#            lTemp.append(lAllObjects[iIndex])
#        if bRightFound:
#            if isinstance(lAllObjects[iIndex], parser.whitespace):
#                lReturn.append(tokens.New(iStart, iStartLine, lTemp))
#            else:
#                lReturn.append(tokens.New(iStart, iStartLine, lTemp[:-1]))
#            bRightFound = False
#            lTemp = []
#            bStore = False
#        if isinstance(lAllObjects[iIndex], oRight) and bStore:
#            if not include_trailing_whitespace:
#                if bExcludeLastToken:
#                    lTemp.pop()
#                    if isinstance(lTemp[-1], parser.whitespace):
#                        lTemp.pop()
#                    if isinstance(lTemp[-1], parser.carriage_return):
#                        lTemp.pop()
#                lReturn.append(tokens.New(iStart, iStartLine, lTemp))
#                lTemp = []
#                bStore = False
#            else:
#                bRightFound = True
#
#        if isinstance(lAllObjects[iIndex], parser.carriage_return):
#            iLine +=1
#
#    return lReturn




from vsg import token

from vsg.rules import whitespace_before_token

lTokens = []
lTokens.append(token.attribute_declaration.colon)


class rule_101(whitespace_before_token):
    '''
    This rule checks for at least a single space before the colon.

    **Violation**

    .. code-block:: vhdl

       attribute max_delay: time;

    **Fix**

    .. code-block:: vhdl

       attribute max_delay : time;
    '''
    def __init__(self):
        whitespace_before_token.__init__(self, 'attribute_declaration', '101', lTokens)




from vsg.rules import single_space_between_tokens

from vsg.token import entity_declaration as token


class rule_002(single_space_between_tokens):
    '''
    This rule checks for a single space after the **entity** keyword.

    **Violation**

    .. code-block:: vhdl

       entity    fifo is

    **Fix**

    .. code-block:: vhdl

       entity fifo is
    '''
    def __init__(self):
        single_space_between_tokens.__init__(self, 'entity', '002', token.entity_keyword, token.identifier)
        self.solution = 'Reduce spaces between *entity* keyword and identifier to a single space.'




from vsg.rules import single_space_between_tokens

from vsg.token import entity_declaration as token


class rule_013(single_space_between_tokens):
    '''
    This rule checks for a single space after the **entity** keyword in the closing of the entity declaration.

    **Violation**

    .. code-block:: vhdl

       end entity    fifo;

    **Fix**

    .. code-block:: vhdl

       end entity fifo;
    '''
    def __init__(self):
        single_space_between_tokens.__init__(self, 'entity', '013', token.end_entity_keyword, token.entity_simple_name)
        self.solution = 'Reduce spaces between *entity* keyword and the entity simple name to a single space.'




from vsg import parser
from vsg import violation

from vsg.rule_group import whitespace
from vsg.rules import utils as rules_utils


class whitespace_before_tokens_in_between_tokens(whitespace.Rule):
    '''
    Checks for at least a single space before a token.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    lTokens : list of token object types
       reference token to check for a whitespace before

    oStart : token object type
       The beginning of the range

    oEnd : token object type
       The end of the range
    '''

    def __init__(self, name, identifier, lTokens, oStart, oEnd):
        whitespace.Rule.__init__(self, name=name, identifier=identifier)
        self.lTokens = lTokens
        self.oStart = oStart
        self.oEnd = oEnd

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_token_and_n_tokens_before_it_in_between_tokens(self.lTokens, 1, self.oStart, self.oEnd)

    def _analyze(self, lToi):
        for oToi in lToi:
            lTokens = oToi.get_tokens()

            if isinstance(lTokens[0], parser.whitespace):
                continue

            if isinstance(lTokens[0], parser.carriage_return):
                continue

            oViolation = violation.New(oToi.get_line_number(), oToi, self.solution)
            self.add_violation(oViolation)

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()
        rules_utils.insert_whitespace(lTokens, 1)
        oViolation.set_tokens(lTokens)





from vsg import parser
from vsg import violation

from vsg.rule_group import whitespace
from vsg.rules import utils as rules_utils

from vsg.vhdlFile import utils


class single_space_between_tokens(whitespace.Rule):
    '''
    Checks for a single space between two tokens.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    left_token : token object
       The left token to check for a space between the right token

    right_token : token object
       The right token to check for a space between the left token
    '''

    def __init__(self, name, identifier, left_token, right_token):
        whitespace.Rule.__init__(self, name=name, identifier=identifier)
        self.left_token = left_token
        self.right_token = right_token

    def _get_tokens_of_interest(self, oFile):
        lToi_a = oFile.get_sequence_of_tokens_matching([self.left_token, parser.whitespace, self.right_token])
        lToi_b = oFile.get_sequence_of_tokens_matching([self.left_token, self.right_token])

        return utils.combine_two_token_class_lists(lToi_a, lToi_b)

    def _analyze(self, lToi):
        for oToi in lToi:
            lTokens = oToi.get_tokens()
            if len(lTokens) == 2:
                self.add_violation(violation.New(oToi.get_line_number(), oToi, self.solution))
            elif len(lTokens[1].get_value()) != 1:
                self.add_violation(violation.New(oToi.get_line_number(), oToi, self.solution))

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()
        if isinstance(lTokens[1], parser.whitespace):
            lTokens[1].set_value(' ')
        else:
            rules_utils.insert_whitespace(lTokens, 1)
        oViolation.set_tokens(lTokens)




from vsg import parser


class New():

    def __init__(self, iStartIndex, iLine, lTokens):

        self.iStartIndex = iStartIndex
        self.lTokens = lTokens
        self.iLine = iLine
        self.sTokenValue = None
        self.iEndIndex = calculate_end_index(iStartIndex, lTokens)

    def get_tokens(self):
        return self.lTokens

    def set_tokens(self, lTokens):
        self.lTokens = lTokens

    def get_line_number(self):
        return self.iLine

    def set_token_value(self, sToken):
        self.sTokenValue = sToken

    def get_token_value(self):
        return self.sTokenValue

    def extract_tokens(self, iStart, iEnd):
        lTokens = self.lTokens[iStart:iEnd + 1]
        iStartIndex = iStart + self.iStartIndex
        iLine = self.iLine
        for iIndex in range(0, iStart + 1):
            try:
                if isinstance(self.lTokens[iIndex], parser.carriage_return):
                    iLine += 1
            except IndexError:
                iLine = iLine
        return New(iStartIndex, iLine, lTokens)

    def get_start_index(self):
        return self.iStartIndex

    def get_end_index(self):
        return self.iEndIndex


def calculate_end_index(iStartIndex, lTokens):
    try:
        iReturn = iStartIndex
        for oToken in lTokens:
            if not isinstance(oToken, parser.beginning_of_file):
                iReturn += 1
        return iReturn
    except TypeError:
        return None




from vsg import token

from vsg.rules import single_space_between_token_pairs

lTokens = []
lTokens.append([token.full_type_declaration.identifier, token.full_type_declaration.is_keyword])


class rule_006(single_space_between_token_pairs):
    '''
    This rule checks for a single space before the **is** keyword.

    **Violation**

    .. code-block:: vhdl

       type state_machine    is (idle, write, read, done);

    **Fix**

    .. code-block:: vhdl

       type state_machine is (idle, write, read, done);
    '''
    def __init__(self):
        single_space_between_token_pairs.__init__(self, 'type', '006', lTokens)
        self.solution = 'Ensure only a single space before the *is* keyword.'




from .rule_001 import rule_001
from .rule_002 import rule_002
from .rule_003 import rule_003
from .rule_004 import rule_004
from .rule_005 import rule_005
from .rule_006 import rule_006
from .rule_007 import rule_007
from .rule_008 import rule_008
from .rule_009 import rule_009
from .rule_010 import rule_010
from .rule_011 import rule_011
from .rule_012 import rule_012
from .rule_013 import rule_013
from .rule_014 import rule_014
from .rule_015 import rule_015

#  ordering rules to handle compressed if statements
from .rule_020 import rule_020  # end if
from .rule_021 import rule_021  # else
from .rule_022 import rule_022  # code after else
from .rule_023 import rule_023  # elsif
from .rule_024 import rule_024  # code after then

from .rule_025 import rule_025
from .rule_026 import rule_026
from .rule_027 import rule_027
from .rule_028 import rule_028
from .rule_029 import rule_029
from .rule_030 import rule_030
from .rule_031 import rule_031
from .rule_032 import rule_032
from .rule_033 import rule_033
from .rule_034 import rule_034
from .rule_035 import rule_035
from .rule_036 import rule_036




from vsg import parser

from vsg.vhdlFile.extract import tokens
from vsg.vhdlFile.extract import utils


def get_tokens_at_beginning_of_line_matching_unless_between_tokens(lTokens, lUnless, lAllTokens, oTokenMap):

    lIndexes = utils.get_indexes_of_token_list(lTokens, oTokenMap)
    lIndexes = filter_indexes_in_unless_regions(lIndexes, lUnless, oTokenMap)

    lReturn = []
    for iIndex in lIndexes:
        if oTokenMap.is_token_at_index(parser.carriage_return, iIndex - 1):
            iLine = oTokenMap.get_line_number_of_index(iIndex)
            lReturn.append(tokens.New(iIndex, iLine, [lAllTokens[iIndex]]))
        elif oTokenMap.is_token_at_index(parser.carriage_return, iIndex - 2) and oTokenMap.is_token_at_index(parser.whitespace, iIndex - 1):
            iLine = oTokenMap.get_line_number_of_index(iIndex)
            lReturn.append(tokens.New(iIndex - 1, iLine, lAllTokens[iIndex - 1: iIndex + 1]))


    return lReturn


def filter_indexes_in_unless_regions(lIndexes, lUnless, oTokenMap):
    lReturn = []

    lUnlessIndexes = utils.get_indexes_of_token_pairs(lUnless, oTokenMap)
    if len(lUnlessIndexes) == 0:
        return lIndexes

    for iIndex in lIndexes:
        bAppend = True
        for unless in lUnlessIndexes:
            if iIndex >= unless[0] and iIndex <= unless[1]:
                bAppend = False
                break
        if bAppend:
            lReturn.append(iIndex)

    return lReturn




from vsg import token
from vsg import violation
from vsg.rule_group import case


class rule_028(case.Rule):
    '''
    This rule checks the entity name has proper case in direct instantiations.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       instance_name : entity library.ENTITY_NAME

    **Fix**

    .. code-block:: vhdl

       instance_name : entity library.entity_name
    '''

    def __init__(self):
        case.Rule.__init__(self, name='instantiation', identifier='028')
        self.case = 'lower'
        self.configuration.append('case')
        self.groups.append('case::name')

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_tokens_matching([token.instantiated_unit.entity_name])

    def _analyze(self, lToi):
        for oToi in lToi:
            lTokens = oToi.get_tokens()
            dAction = {}
            if '.' in lTokens[0].get_value():
                lTemp = lTokens[0].get_value().split('.')
                sObjectValue = lTemp[-1]
                dAction['period'] = True
            else:
                sObjectValue = lTokens[0].get_value()
                dAction['period'] = False

            if self.case == 'lower':
                if not sObjectValue.islower():
                    sSolution = 'Change "' + sObjectValue + '" to "' + sObjectValue.lower() + '"'
                    oViolation = violation.New(oToi.get_line_number(), oToi, sSolution)
                    oViolation.set_action(dAction)
                    self.add_violation(oViolation)
            if self.case == 'upper':
                if not sObjectValue.isupper():
                    sSolution = 'Change "' + sObjectValue + '" to "' + sObjectValue.upper() + '"'
                    oViolation = violation.New(oToi.get_line_number(), oToi, sSolution)
                    oViolation.set_action(dAction)
                    self.add_violation(oViolation)

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()
        dAction = oViolation.get_action()
        if not dAction['period']:
            if self.case == 'lower':
                lTokens[0].set_value(lTokens[0].get_value().lower())
            if self.case == 'upper':
                lTokens[0].set_value(lTokens[0].get_value().upper())
        else:
            lTemp = lTokens[0].get_value().split('.')
            if self.case == 'lower':
                lTemp[-1] = lTemp[-1].lower()
            if self.case == 'upper':
                lTemp[-1] = lTemp[-1].upper()
            lTokens[0].set_value('.'.join(lTemp))
        oViolation.set_tokens(lTokens)




from vsg.rules import token_case_with_prefix_suffix

from vsg import token

lTokens = []
lTokens.append(token.package_declaration.is_keyword)


class rule_013(token_case_with_prefix_suffix):
    '''
    This rule checks the **is** keyword has proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       package fifo_pkg IS

    **Fix**

    .. code-block:: vhdl

       package fifo_pkg is
    '''

    def __init__(self):
        token_case_with_prefix_suffix.__init__(self, 'package', '013', lTokens)
        self.groups.append('case::keyword')




from vsg import parser
from vsg import violation

from vsg.token import assertion
from vsg.token import assertion_statement
from vsg.token import concurrent_assertion_statement

from vsg.rules import utils as rules_utils
from vsg.rule_group import alignment
from vsg.vhdlFile import utils


class rule_400(alignment.Rule):
    '''
    This rule checks the alignment of the report expressions.

    .. NOTE:: There is a configuration option **alignment** which changes the indent location of multiple lines.

    alignment set to 'report' (Default)
    ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

    **Violation**

    .. code-block:: vhdl

       assert WIDTH > 16
         report "FIFO width is limited" &
       " to 16 bits."
         severity FAILURE;

    **Fix**

    .. code-block:: vhdl

       assert WIDTH > 16
         report "FIFO width is limited" &
                " to 16 bits."
         severity FAILURE;

    alignment set to 'left'
    ^^^^^^^^^^^^^^^^^^^^^^^

    **Violation**

    .. code-block:: vhdl

       assert WIDTH > 16
         report "FIFO width is limited" &
       " to 16 bits."
         severity FAILURE;

    **Fix**

    .. code-block:: vhdl

       assert WIDTH > 16
         report "FIFO width is limited" &
           " to 16 bits."
         severity FAILURE;
    '''

    def __init__(self):
        alignment.Rule.__init__(self, name="assert", identifier="400")
        self.alignment = 'report'
        self.configuration.append('alignment')
        self.phase = 4

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_tokens_starting_with_token_and_ending_with_one_of_possible_tokens([assertion.report_keyword], [assertion.severity_keyword, assertion_statement.semicolon, concurrent_assertion_statement.semicolon], True, False, True)

    def analyze(self, oFile):

        self._print_debug_message('Analyzing rule: ' + self.unique_id)
        lToi = self._get_tokens_of_interest(oFile)

        for oToi in lToi:
            iLine, lTokens = utils.get_toi_parameters(oToi)

            iSpaces = self._calculate_column(oFile, oToi, lTokens)

            for iToken, oToken in enumerate(lTokens):
                if isinstance(oToken, parser.carriage_return):
                    iLine += 1
                    if isinstance(lTokens[iToken + 1], parser.whitespace):
                        if len(lTokens[iToken + 1].get_value()) != iSpaces:
                            self._create_violation(iLine, iSpaces, 'adjust', oToi.extract_tokens(iToken + 1, iToken + 1))
                    elif isinstance(lTokens[iToken + 1], parser.blank_line):
                            continue
                    else:
                        self._create_violation(iLine, iSpaces, 'insert', oToi.extract_tokens(iToken + 1, iToken + 1))

    def fix(self, oFile, dFixOnly=None):
        '''
        Applies fixes for any rule violations.
        '''
        if self.fixable:
            self.analyze(oFile)
            self._print_debug_message('Fixing rule: ' + self.unique_id)
            self._filter_out_fix_only_violations(dFixOnly)
            for oViolation in self.violations[::-1]:
                self._fix_violation(oViolation)
            oFile.update(self.violations)
            self.clear_violations()

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()
        dAction = oViolation.get_action()

        if dAction['action'] == 'adjust':
            lTokens[0].set_value(' '*dAction['column'])
        else:
            rules_utils.insert_whitespace(lTokens, 0, dAction['column'])

        oViolation.set_tokens(lTokens)

    def _create_violation(self, iLine, iSpaces, sAction, lTokens):
        sSolution = 'Indent line to column ' + str(iSpaces)
        oViolation = violation.New(iLine, lTokens, sSolution)
        dAction = {}
        dAction['action'] = sAction
        dAction['column'] = iSpaces
        oViolation.set_action(dAction)
        self.add_violation(oViolation)

    def _calculate_column(self, oFile, oToi, lTokens):
        if self.alignment == 'report':
            iSpaces = oFile.get_column_of_token_index(oToi.get_start_index()) + 7
        else:
            iSpaces = (lTokens[0].indent + 1) * self.indentSize
        return iSpaces




from vsg import parser
from vsg import violation

from vsg.rules import utils as rules_utils
from vsg.rule_group import alignment


class align_consecutive_lines_after_line_starting_with_token_and_stopping_with_token(alignment.Rule):
    '''
    Checks for a single space between two tokens.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    start_token : token object
       reference token to align comments with

    stop_token : token object
       The token that ends the region to perform the alignment
    '''

    def __init__(self, name, identifier, start_token, stop_token):
        alignment.Rule.__init__(self, name=name, identifier=identifier)
        self.phase = 4
        self.start_token = start_token
        self.stop_token = stop_token

    def analyze(self, oFile):

        lToi = oFile.get_tokens_where_line_starts_with_token_until_ending_token_is_found(self.start_token, self.stop_token)
        for oToi in lToi:
            lTokens = oToi.get_tokens()
            iLine = oToi.get_line_number()

            for iIndex in range(0, len(lTokens)):
               oToken = lTokens[iIndex]

               if isinstance(oToken, self.start_token):
                   iTargetIndent = oToken.get_indent()
                   iWhitespaceLength = iTargetIndent * self.indentSize + len(oToken.get_value()) + 1

               if isinstance(oToken, self.stop_token):
                   break

               if isinstance(oToken, parser.carriage_return):
                   iLine += 1
                   if isinstance(lTokens[iIndex + 1], parser.whitespace):
                       if len(lTokens[iIndex + 1].get_value()) != iWhitespaceLength:
                           oLineTokens = oFile.get_tokens_from_line(iLine)
                           oViolation = violation.New(iLine, oLineTokens, self.solution)
                           dAction = {}
                           dAction['type'] = 'adjust'
                           dAction['adjust'] = iWhitespaceLength
                           oViolation.set_action(dAction)
                           self.add_violation(oViolation)
                   else:
                       oLineTokens = oFile.get_tokens_from_line(iLine)
                       oViolation = violation.New(iLine, oLineTokens, self.solution)
                       dAction = {}
                       dAction['type'] = 'insert'
                       dAction['adjust'] = iWhitespaceLength
                       oViolation.set_action(dAction)
                       self.add_violation(oViolation)

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()
        dAction = oViolation.get_action()
        if dAction['type'] == 'adjust':
            lTokens[0].set_value(' ' * dAction['adjust'])
        elif dAction['type'] == 'insert':
            rules_utils.insert_whitespace(lTokens, 0, dAction['adjust'])
        oViolation.set_tokens(lTokens)




import bisect

from vsg.vhdlFile.extract import tokens


def get_tokens_between_tokens_inclusive_while_storing_value_from_token(left_token, right_token, value_token, lAllTokens, oTokenMap):

    lReturn = []

    lStart, lEnd = oTokenMap.get_token_pair_indexes(left_token, right_token)

    lValueIndexes = oTokenMap.get_token_indexes(value_token)

    iPreviousEnd = 0
    for iStart, iEnd in zip(lStart, lEnd):
        sValue = None

        iValueIndex = bisect.bisect_left(lValueIndexes, iEnd) - 1
        if iValueIndex >= 0:
            iValue = lValueIndexes[iValueIndex]

            oValueToken = lAllTokens[iValue]
            if iValue < iPreviousEnd:
                sValue = None
            else:
                sValue = oValueToken.get_value()
        else:
            sValue = None

        iLine = oTokenMap.get_line_number_of_index(iStart)

        oTokens = tokens.New(iStart, iLine, lAllTokens[iStart:iEnd + 1])
        oTokens.set_token_value(sValue)
        lReturn.append(oTokens)
        iPreviousEnd = iEnd

    return lReturn




from vsg import parser


class New():

  def __init__(self):
      self.code_tags = []
      self.next_line_code_tags = []
      self.bIgnoreNextCarriageReturn = False
      self.iLine = 0

  def clear(self):
      self.code_tags.clear()
      self.next_line_code_tags.clear()

  def remove(self, sCodeTag):
      self.code_tags.remove(sCodeTag)

  def add(self, sCodeTag):
      if sCodeTag not in self.code_tags:
          self.code_tags.append(sCodeTag)

  def get_tags(self):
      lReturn = []
      lReturn.extend(self.code_tags)
      lReturn.extend(self.next_line_code_tags)
      return lReturn

  def update(self, oToken):
      if isinstance(oToken, parser.comment):
          sValue = oToken.get_value()

          if sValue.startswith('-- vsg_on'):
              lValues = sValue.split()
              if len(lValues) == 2:
                  self.clear()
              else:
                 for sCodeTag in lValues[2:]:
                     self.remove(sCodeTag)
          elif sValue.startswith('-- vsg_off'):
              lValues = sValue.split()
              if len(lValues) == 2:
                  self.clear()
                  self.add('all')
              else:
                 for sCodeTag in lValues[2:]:
                     self.add(sCodeTag)
          elif sValue.startswith('-- vsg_disable_next_line'):
              lValues = sValue.split()
              for sCodeTag in lValues[2:]:
                 if sCodeTag not in self.next_line_code_tags:
                     self.next_line_code_tags.append(sCodeTag)
              self.bIgnoreNextCarriageReturn = True
      elif isinstance(oToken, parser.carriage_return):
          self.iLine += 1
          if self.bIgnoreNextCarriageReturn:
              self.bIgnoreNextCarriageReturn = False
          else:
              self.next_line_code_tags.clear()




from vsg.token import conditional_force_assignment as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import force_mode
from vsg.vhdlFile.classify import conditional_expressions


def detect(iToken, lObjects):
    '''
    conditional_force_assignment ::=
        target <= force [ force_mode ] conditional_expressions ;
    '''

    if utils.is_next_token_one_of(['when', 'if', 'elsif', 'else'], iToken, lObjects):
        return False
    if utils.find_in_range('<=', iToken, ';', lObjects):#
        if utils.find_in_range('force', iToken, ';', lObjects):
            return classify(iToken, lObjects)
    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.assign_tokens_until('<=', token.target, iToken, lObjects)
    iCurrent = utils.assign_next_token_required('<=', token.assignment, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required('force', token.force_keyword, iCurrent, lObjects)

    iCurrent = force_mode.detect(iCurrent, lObjects)

    iCurrent = conditional_expressions.classify_until([';'], iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    return iCurrent




from vsg.rules import single_space_between_tokens

from vsg.token import architecture_body as token


class rule_032(single_space_between_tokens):
    '''
    This rule checks for a single space between the **of** keyword and the entity_name.

    **Violation**

    .. code-block:: vhdl

       architecture rtl of    fifo is

    **Fix**

    .. code-block:: vhdl

       architecture rtl of fifo is
    '''
    def __init__(self):
        single_space_between_tokens.__init__(self, 'architecture', '032', token.of_keyword, token.entity_name)
        self.solution = 'Reduce spaces between *of* keyword and entity_name to a single space.'




from vsg.rules import remove_excessive_blank_lines_below_line_ending_with_token

from vsg import token

lTokens = []
lTokens.append(token.if_statement.then_keyword)

lOverrides = []
lOverrides.append(token.case_statement.case_label)
lOverrides.append(token.case_statement.case_keyword)


class rule_006(remove_excessive_blank_lines_below_line_ending_with_token):
    '''
    This rule checks for blank lines after the **then** keyword.

    **Violation**

    .. code-block:: vhdl

      if (a = '1') then


        b <= '0'

    **Fix**

    .. code-block:: vhdl

      if (a = '1') then
        b <= '0'
    '''
    def __init__(self):
        remove_excessive_blank_lines_below_line_ending_with_token.__init__(self, 'if', '006', lTokens, iAllow=0, lOverrides=lOverrides)




from vsg.rules import token_case

from vsg import token

lTokens = []
lTokens.append(token.function_specification.designator)


class rule_017(token_case):
    '''
    This rule checks the function designator has proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       function OVERflow (a: integer) return integer is

    **Fix**

    .. code-block:: vhdl

       function overflow (a: integer) return integer is
    '''

    def __init__(self):
        token_case.__init__(self, 'function', '017', lTokens)
        self.groups.append('case::name')




from vsg import parser

from vsg.vhdlFile.extract import tokens


def get_tokens_bounded_by_token_when_between_tokens(oLeft, oRight, oStart, oEnd, lAllTokens, oTokenMap, include_trailing_whitespace=False):

    lLeft, lRight = oTokenMap.get_token_pair_indexes(oLeft, oRight)
    lStart, lEnd = oTokenMap.get_token_pair_indexes(oStart, oEnd)
    lReturn = []
    for iStart, iEnd in zip(lStart, lEnd):
        for iLeft, iRight in zip(lLeft, lRight):
            if iStart < iLeft and iRight < iEnd:
                iLine = oTokenMap.get_line_number_of_index(iLeft)
                if include_trailing_whitespace:
                    if oTokenMap.is_token_at_index(iRight + 1, parser.whitespace):
                        lReturn.append(tokens.New(iLeft, iLine, lAllTokens[iLeft:iRight + 2]))
                    else:
                        lReturn.append(tokens.New(iLeft, iLine, lAllTokens[iLeft:iRight + 1]))
                else:
                    lReturn.append(tokens.New(iLeft, iLine, lAllTokens[iLeft:iRight + 1]))
    return lReturn




from vsg import token

from vsg.rules import remove_carriage_returns_between_token_pairs

lTokens = []
lTokens.append([token.signal_declaration.signal_keyword, token.signal_declaration.semicolon])


class rule_016(remove_carriage_returns_between_token_pairs):
    '''
    This rule checks the signal declaration is on a single line.

    **Violation**

    .. code-block:: vhdl

       signal sig1
         : std_logic;

       signal sig2 :
         std_logic;

    **Fix**

    .. code-block:: vhdl

       signal sig1 : std_logic;

       signal sig2 : std_logic;
    '''

    def __init__(self):
        remove_carriage_returns_between_token_pairs.__init__(self, 'signal', '016', lTokens)
        self.solution = 'ensure signal declaration is on a single line.'




from vsg import parser
from vsg import token

from vsg.rules import single_space_between_token_pairs

lTokens = []
lTokens.append([token.if_statement.elsif_keyword, parser.open_parenthesis])


class rule_005(single_space_between_token_pairs):
    '''
    This rule checks for a single space between the **elsif** keyword and the (.

    **Violation**

    .. code-block:: vhdl

      elsif(c = '1') then

      elsif   (c = '1') then

    **Fix**

    .. code-block:: vhdl

      elsif (c = '1') then

      elsif (c = '1') then
    '''
    def __init__(self):
        single_space_between_token_pairs.__init__(self, 'if', '005', lTokens)
        self.solution = 'Ensure only a single space exists between the *elsif* keyword and {.'




from vsg.rules import remove_excessive_blank_lines_below_line_ending_with_token

from vsg.token import context_declaration as token


class rule_026(remove_excessive_blank_lines_below_line_ending_with_token):
    '''
    This rule ensures a single blank line after the **context** keword.

    **Violation**

    .. code-block:: vhdl

       context c1 is



         library ieee;

    **Fix**

    .. code-block:: vhdl

       context c1 is

         library ieee;
    '''
    def __init__(self):
        remove_excessive_blank_lines_below_line_ending_with_token.__init__(self, 'context', '026', [token.is_keyword])




from vsg.rules import single_space_between_tokens

from vsg.token import component_declaration as token


class rule_013(single_space_between_tokens):
    '''
    This rule checks for a single space after the **component** keyword in the **end component** line.

    **Violation**

    .. code-block:: vhdl

       end component    fifo;

    **Fix**

    .. code-block:: vhdl

       end component fifo;
    '''
    def __init__(self):
        single_space_between_tokens.__init__(self, 'component', '013', token.end_component_keyword, token.component_simple_name)
        self.solution = 'Reduce spaces between *end* keyword and *component* keyword.'




from vsg.rules import single_space_between_tokens

from vsg.token import architecture_body as token


class rule_033(single_space_between_tokens):
    '''
    This rule checks for a single space between the entity_name and the **is** keyword.

    **Violation**

    .. code-block:: vhdl

       architecture rtl of fifo    is

    **Fix**

    .. code-block:: vhdl

       architecture rtl of fifo is
    '''
    def __init__(self):
        single_space_between_tokens.__init__(self, 'architecture', '033', token.entity_name, token.is_keyword)
        self.solution = 'Reduce spaces between the entity_name and the *is* keyword to a single space.'




from vsg.rules import remove_excessive_blank_lines_above_line_starting_with_token

from vsg.token import context_declaration as token


class rule_027(remove_excessive_blank_lines_above_line_starting_with_token):
    '''
    This rule ensures a single blank line before the **end** keword.

    **Violation**

    .. code-block:: vhdl

         use ieee.std_logic_1164.all;



       end context;

    **Fix**

    .. code-block:: vhdl

         use ieee.std_logic_1164.all;

       end context;
    '''
    def __init__(self):
        remove_excessive_blank_lines_above_line_starting_with_token.__init__(self, 'context', '027', [token.end_keyword])




from vsg.rules import align_consecutive_lines_starting_with_a_comment_above_line_starting_with_token

from vsg.token import if_statement as token


class rule_032(align_consecutive_lines_starting_with_a_comment_above_line_starting_with_token):
    '''
    This rule aligns consecutive comment only lines above the **elsif** keyword in if statements.
    These comments are used to describe what the elsif code is going to do.

    **Violation**

    .. code-block:: vhdl

           -- comment 1
     -- comment 2
        -- comment 3
       elsif (a = '1')
         rd_en <= '0';

    **Fix**

    .. code-block:: vhdl

       -- comment 1
       -- comment 2
       -- comment 3
       elsif (a = '1')
         rd_en <= '0';
    '''

    def __init__(self):
        align_consecutive_lines_starting_with_a_comment_above_line_starting_with_token.__init__(self, 'if', '032', token.elsif_keyword)
        self.solution = 'Align comment with *elsif* keyword.'




from vsg.rules import single_space_between_tokens

from vsg.token import loop_statement as token


class rule_103(single_space_between_tokens):
    '''
    This rule checks if a label exists that a single space exists between the label and the colon.

    **Violation**

    .. code-block:: vhdl

         label: for index in 4 to 23 loop
         label    : for index in 0 to 100 loop

    **Fix**

    .. code-block:: vhdl

         label : for index in 4 to 23 loop
         label : for index in 0 to 100 loop
    '''
    def __init__(self):
        single_space_between_tokens.__init__(self, 'loop_statement', '103', token.loop_label, token.label_colon)
        self.solution = 'Ensure a single space between label and :.'
